@@ -14,7 +14,7 @@
 import os
 import math
 from enum import Enum
-from typing import Any
+from typing import Any, Optional
 
 import torch
 
@@ -36,11 +36,12 @@ EPSILON_FP16 = 1e-5
 
 class Accelerator(object):
 
-    def __init__(self, trainer=None, cluster_environment=None):
+    def __init__(self, trainer=None, cluster_environment=None, ddp_plugin=None):
         self.trainer = trainer
         self.nickname = None
         self.cluster_environment = cluster_environment
         self.dist = AttributeDict(rank=0, device=None)
+        self.ddp_plugin = ddp_plugin
 
         if trainer is not None:
             self.train_loop = self.trainer.train
@@ -51,9 +52,10 @@ class Accelerator(object):
         pass
 
     def teardown(self):
-        pass
+        # Ensure if necessary all processes are finished
+        self.barrier()
 
-    def barrier(self, name: str = None):
+    def barrier(self, name: Optional[str] = None):
         pass
 
     def broadcast(self, obj, src=0):
@@ -91,11 +93,8 @@ class Accelerator(object):
             )
         else:
             # do backward pass
-            if self.trainer.train_loop.automatic_optimization:
-                model = self.trainer.get_model()
-                model.backward(closure_loss, optimizer, opt_idx)
-            else:
-                closure_loss.backward(*args, **kwargs)
+            model = self.trainer.get_model()
+            model.backward(closure_loss, optimizer, opt_idx, *args, **kwargs)
 
             # once backward has been applied, release graph
             closure_loss = closure_loss.detach()
@@ -219,7 +218,8 @@ class Accelerator(object):
             'trainer': self.trainer,
             'nickname': self.nickname,
             'cluster_environment': self.cluster_environment,
-            'dist': self.dist
+            'dist': self.dist,
+            'ddp_plugin': self.ddp_plugin
         }
 
     def __setstate__(self, d):
@@ -227,6 +227,7 @@ class Accelerator(object):
         self.nickname = d['nickname']
         self.cluster_environment = d['cluster_environment']
         self.dist = d['dist']
+        self.ddp_plugin = d['ddp_plugin']
 
 
 # TODO: allow user to compare with string even internaly we shall use these Enum to prevent typos...
