@@ -29,7 +29,7 @@ class Result(Dict):
         self,
         minimize: Optional[Tensor] = None,
         early_stop_on: Optional[Tensor] = None,
-        checkpoint_on: Union[Tensor, bool, None] = None,
+        checkpoint_on: Optional[Union[Tensor, bool]] = None,
         hiddens: Optional[Tensor] = None,
     ):
 
@@ -633,8 +633,8 @@ class TrainResult(Result):
     def __init__(
         self,
         minimize: Optional[Tensor] = None,
-        early_stop_on: Tensor = None,
-        checkpoint_on: Union[Tensor, bool] = None,
+        early_stop_on: Optional[Tensor] = None,
+        checkpoint_on: Optional[Union[Tensor, bool]] = None,
         hiddens: Optional[Tensor] = None,
     ):
         """
