@@ -12,9 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import torch
-from typing import Any, Callable, Optional, Union
+from typing import Any, Optional
 
 from pytorch_lightning.metrics.metric import Metric
+from pytorch_lightning.metrics.functional.mean_squared_log_error import (
+    _mean_squared_log_error_update,
+    _mean_squared_log_error_compute
+)
 
 
 class MeanSquaredLogError(Metric):
@@ -64,14 +68,13 @@ class MeanSquaredLogError(Metric):
             preds: Predictions from model
             target: Ground truth values
         """
-        self._check_same_shape(preds, target)
-        squared_log_error = torch.pow(torch.log1p(preds) - torch.log1p(target), 2)
+        sum_squared_log_error, n_obs = _mean_squared_log_error_update(preds, target)
 
-        self.sum_squared_log_error += torch.sum(squared_log_error)
-        self.total += target.numel()
+        self.sum_squared_log_error += sum_squared_log_error
+        self.total += n_obs
 
     def compute(self):
         """
         Compute mean squared logarithmic error over state.
         """
-        return self.sum_squared_log_error / self.total
+        return _mean_squared_log_error_compute(self.sum_squared_log_error, self.total)
