@@ -14,7 +14,7 @@
 import inspect
 import os
 from argparse import ArgumentParser, Namespace
-from typing import Union, List, Tuple, Any
+from typing import Dict, Union, List, Tuple, Any
 from pytorch_lightning.utilities import parsing
 
 
@@ -160,7 +160,7 @@ def add_argparse_args(cls, parent_parser: ArgumentParser) -> ArgumentParser:
 
     allowed_types = (str, int, float, bool)
 
-    # TODO: get "help" from docstring :)
+    args_help = parse_args_from_docstring(cls.__init__.__doc__ or cls.__doc__)
     for arg, arg_types, arg_default in (
         at for at in get_init_arguments_and_types(cls) if at[0] not in depr_arg_names
     ):
@@ -174,8 +174,7 @@ def add_argparse_args(cls, parent_parser: ArgumentParser) -> ArgumentParser:
             # if the only arg type is bool
             if len(arg_types) == 1:
                 use_type = parsing.str_to_bool
-            # if only two args (str, bool)
-            elif len(arg_types) == 2 and set(arg_types) == {str, bool}:
+            elif str in arg_types:
                 use_type = parsing.str_to_bool_or_str
             else:
                 # filter out the bool as we need to use more general
@@ -200,13 +199,36 @@ def add_argparse_args(cls, parent_parser: ArgumentParser) -> ArgumentParser:
             dest=arg,
             default=arg_default,
             type=use_type,
-            help='autogenerated by pl.Trainer',
+            help=args_help.get(arg),
             **arg_kwargs,
         )
 
     return parser
 
 
+def parse_args_from_docstring(docstring: str) -> Dict[str, str]:
+    arg_block_indent = None
+    current_arg = None
+    parsed = {}
+    for line in docstring.split("\n"):
+        stripped = line.lstrip()
+        if not stripped:
+            continue
+        line_indent = len(line) - len(stripped)
+        if stripped.startswith(('Args:', 'Arguments:', 'Parameters:')):
+            arg_block_indent = line_indent + 4
+        elif arg_block_indent is None:
+            continue
+        elif line_indent < arg_block_indent:
+            break
+        elif line_indent == arg_block_indent:
+            current_arg, arg_description = stripped.split(':', maxsplit=1)
+            parsed[current_arg] = arg_description.lstrip()
+        elif line_indent > arg_block_indent:
+            parsed[current_arg] += f' {stripped}'
+    return parsed
+
+
 def _gpus_allowed_type(x) -> Union[int, str]:
     if ',' in x:
         return str(x)
